import {
  Http, IDictionary, Loader, Metadata,
} from '@zeedhi/core';
import {
  Button, DialogService, Form, Image, LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, TextInput,
} from '@zeedhi/common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import { Utilities } from '@/util/Utilities';
import ColorPicker from '@/components/colorPicker/ZdColorPicker';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00002LocalController {
    private modalNovoLocal!: Modal;

    private modalEditarLocal!: Modal;

    private modalAlterarCor!: Modal;

    private modalQrCode!: Modal;

    private modalConfigurarLocais!: Modal;

    public modoAberturaModal: boolean = false;

    private componente: any;

    private CDCLIENTE: any;

    public qrCodeController = Loader.getInstance('QRCodeController');

    private clicks = 0;

    public visibilidadeGridConfigurarLocais: boolean = false;

    public async abrirModal({ component }: any) {
      this.componente = component;
      this.modalNovoLocal = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_00002_001_modal_novo_local', true);
        if (component.name.split('_')[0] !== 'botao-acionar-adicao') {
            this.modoAberturaModal = true;
        } else {
            this.modoAberturaModal = false;
        }
        await this.modalNovoLocal.show();
    }

    public fecharModal() {
        this.modalNovoLocal.hide();
        this.modalNovoLocal?.destroy();
    }

    // função que simula um double click
    public rowClickFunction({ row }: any) {
      this.clicks += 1;
      if (this.clicks >= 2) {
        this.abrirModalEdicao(row);
        this.clicks = 0;
      }
      setTimeout(() => {
        this.clicks = 0;
      }, 500);
    }

    public async abrirModalEdicao(row : any) {
      this.modalEditarLocal = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_00002_001_modal_editar_local', true);
      await this.modalEditarLocal.show();
      this.montarInformacoesEdicao(row);
    }

    public fecharModalEdicao() {
        this.modalEditarLocal.hide();
        this.modalEditarLocal?.destroy();
    }

    public montarInformacoesEdicao(currentRow: any) {
      const form = Metadata.getInstance<Form>('form-editar-local');

      form.value.CDCLIENTE = currentRow.CDCLIENTE;
      form.value.NMLOCALCLIENTE = currentRow.NMLOCALCLIENTE;
      form.value.NMABREVLOCALCLI = currentRow.NMABREVLOCALCLI;
      form.value.NRTIPOLOCALCLI = currentRow.NRTIPOLOCALCLI;
      if (currentRow.NRLOCALCLIENTEPAI != null) {
        form.value.NRLOCALCLIENTE = currentRow.NRLOCALCLIENTEPAI;
      }
      form.value.NRCHECKLIST = currentRow.NRCHECKLIST;
      form.value.DSCODIGOLOCALCLI = currentRow.DSCODIGOLOCALCLI;
      form.value.NRSETOR = currentRow.NRSETOR;
      form.value.nrlocalcliente = currentRow.NRLOCALCLIENTE;
    }

    public atualizarLocal() {
      const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-clientes');
      const form = Metadata.getInstance<Form>('form-editar-local');
      const tipoDeLocal = form.value.NRTIPOLOCALCLI;
      const localPai = form.value.NRLOCALCLIENTE;

      if (form.validate()) {
        Http.post('/atualizarLocal', {
          NRLOCALCLIENTE: form.value.nrlocalcliente,
          NMLOCALCLIENTE: form.value.NMLOCALCLIENTE,
          CDCLIENTE: form.value.CDCLIENTE,
          NRTIPOLOCALCLI: tipoDeLocal.NRTIPOLOCALCLI,
          NRLOCALCLIENTEPAI: (localPai !== null && Object.keys(localPai).includes('NRTIPOLOCALCLI')) ? localPai.NRLOCALCLIENTE : null,
          NRCHECKLISTPADR: form.value.NRCHECKLIST,
          NMABREVLOCALCLI: form.value.NMABREVLOCALCLI,
          DSCODIGOLOCALCLI: form.value.DSCODIGOLOCALCLI,
        }).then(() => {
          Utilities.showSuccessMessage();
          this.fecharModalEdicao();
          cards.datasource.get();
        }).catch((error: any) => {
          Utilities.showErrorMessage(error);
        }).finally(() => {
          LoadingService.hide();
        });
      } else {
        Utilities.alertMessage('Preencha todos os campos obrigatórios (*)', 'error');
      }
    }

    public async abrirModalCor({ component }: any) {
      this.modalAlterarCor = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_00002_001_modal_alterar_cor_cliente', true);
      const valor = component.name.split('_')[1];
      this.CDCLIENTE = valor;
      await this.modalAlterarCor.show();
    }

    public fecharModalCor() {
      this.modalAlterarCor.hide();
      this.modalAlterarCor?.destroy();
    }

    public alterarCorCliente() {
      const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-clientes');

      LoadingService.show();
      Http.post('/alterarCorCliente', {
        CDCLIENTE: this.CDCLIENTE,
        DSCORIDLOCAL: Metadata.getInstance<ColorPicker>('color-picker').value,
      }).then(() => {
        Utilities.showSuccessMessage();
        cards.datasource.get();
        this.fecharModalCor();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }

    public async abrirModalQrCode({ row }: any) {
      this.modalQrCode = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_00002_001_modal_qrCode', true);
      await this.modalQrCode.show();
      LoadingService.show();
      Metadata.getInstance<Image>('qRCodeImagem').src = await this.qrCodeController.generateQRCode(row.NRLOCALCLIENTE);
      Metadata.getInstance<TextInput>('tipoLocalAtendimento').value = row.NMTIPOLOCALCLI;
      Metadata.getInstance<TextInput>('localAtendimento').value = row.NMLOCALCLIENTE;
      LoadingService.hide();
    }

    public fecharModalQrCode() {
      this.modalQrCode.hide();
      this.modalQrCode?.destroy();
    }

    public qrCodeVisivel({ row }: any) {
      return (row.IDTIPOLOCALCLI === '3');
    }

    public visibilidade() {
      const arrayTabs = Metadata.getInstance<Tabs>('tabs-gerenciar-locais');
      const botao = Metadata.getInstance<Button>('botao-acionar-adicao');
      const botaoConfigurarLocais = Metadata.getInstance<Button>('botao-configurar-locais');

      if (arrayTabs.tabs[arrayTabs.activeTab].name === 'tab-tipos-de-local') {
          botao.isVisible = false;
          botaoConfigurarLocais.isVisible = false;
      } else if (arrayTabs.tabs[arrayTabs.activeTab].name === 'tab-locais') {
          botao.isVisible = true;
          botaoConfigurarLocais.isVisible = true;
      }
    }

    public controleAdicaoPorCliente() {
      const selectCliente = Metadata.getInstance<Select>('CDCLIENTE');
      const colorPicker = Metadata.getInstance<ColorPicker>('COLORPICKER');

      selectCliente.disabled = this.modoAberturaModal;
      colorPicker.isVisible = !this.modoAberturaModal;

      this.montarInformacoesModal();
    }

    public montarInformacoesModal() {
      if (this.componente.component === 'TekTreeGrid') {
        const row = this.componente.datasource.currentRow;
        const CDCLIENTE = this.componente.name.split('_')[1];
        const form = Metadata.getInstance<Form>('form-novo-local');

        form.value.CDCLIENTE = CDCLIENTE;

        if (row.NRLOCALCLIENTE) {
          form.value.NRLOCALCLIENTE = row.NRLOCALCLIENTE;
          form.value.DSCORIDLOCAL = row.DSCORIDLOCAL;
        }
      }
    }

    public controleTipoDeLocal({ component }: any) {
        const componentTipo = Metadata.getInstance<Select>('NRTIPOLOCALCLI');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        let id: any;

        if (component.value) {
            id = component.value.IDTIPOLOCALCLI;
        } else {
            id = 0;
            checklist.disabled = true;
            checklist.value = '';
        }

        this.setFitroSelect({ component: componentTipo }, id);
    }

    public setFitroSelect({ component }: any, tipo: any = 0) {
        component.datasource.setFilter({
            IDTIPOLOCALCLI: tipo,
        });
        component.datasource.get();
    }

    public changeTipoDeLocal({ component }: any) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        if (component.value) {
            if (component.value.IDTIPOLOCALCLI === '1') {
                local.disabled = true;
                local.removeValidation('required');
            } else if (component.value.IDTIPOLOCALCLI === '3') {
                checklist.disabled = false;
            }
        } else {
            local.disabled = false;
            local.addValidation('required');
            checklist.disabled = true;
            checklist.value = '';
        }
    }

    public inserirNovoLocal() {
      const tipoDeLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI');
      const localPai = Metadata.getInstance<Select>('NRLOCALCLIENTE');
      const form = Metadata.getInstance<Form>('form-novo-local').validate();
      const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-clientes');
      const valores = localPai.selectValue;

      if (form) {
          LoadingService.show();
          Http.post('/inserirNovoLocal', {
              NMLOCALCLIENTE: Metadata.getInstance<TextInput>('NMLOCALCLIENTE').value,
              CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
              NRTIPOLOCALCLI: tipoDeLocal.value.NRTIPOLOCALCLI,
              NRLOCALCLIENTEPAI: (localPai.value !== null && Object.keys(localPai.value).includes('NRTIPOLOCALCLI')) ? localPai.value.NRLOCALCLIENTE : null,
              NRCHECKLISTPADR: Metadata.getInstance<Select>('NRCHECKLIST').value,
              NMABREVLOCALCLI: Metadata.getInstance<TextInput>('NMABREVLOCALCLI').value,
              DSCODIGOLOCALCLI: Metadata.getInstance<Select>('DSCODIGOLOCALCLI').value,
              DSCORIDLOCAL: Metadata.getInstance<ColorPicker>('COLORPICKER').value,
              NRESTRUTURAM: valores.NRESTRUTURAM,
          }).then(() => {
              Utilities.showSuccessMessage();
              this.fecharModal();
              cards.datasource.get();
          }).catch((error: any) => {
              Utilities.showErrorMessage(error);
          }).finally(() => {
              LoadingService.hide();
          });
      } else {
          Utilities.alertMessage('Preencha todos os campos obrigatórios disponíveis (*)!', 'error');
      }
    }

    public setFiltroTreeGrid({ component }: any) {
      component.datasource.setFilter({
        CDCLIENTE: component.name.split('_')[1],
      });
    }

    public padraoCodigo() {
      Http.get('/paramMascara').then((response: any) => {
        window.localStorage.setItem('padraoMascara', JSON.stringify(response.data[0]));
      });
    }

    public obrigatoriedadeCodigo({ component }: any) {
      Http.get('/paramMascara').then((response: IDictionary) => {
        const idObrigatoriedade = response.data.IDOBRIGACDLOCAT;

        if (idObrigatoriedade === 'S') {
          component.addValidation('required');
          component.cssClass += ' zd-input-required';
        } else {
          component.removeValidation('required');
        }
      });
    }

    public dialogInativarInformacoes({ component, event }: any) {
      event.preventDefault();

      DialogService.show({
        name: `dialog-inativar-${component.name}`,
        type: 'warning',
        title: 'Atenção',
        text: 'Obs: Os locais de atendimento que estiverem em uso não poderão ser inativados. Deseja continuar?',
        buttons: [
          {
            name: 'NAO',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: `sim-${component.name}`,
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                this.inativarLocais({ component });
                DialogService.hide();
              },
            },
          },
        ],
      });
    }

    public async inativarLocais({ component }: any) {
      const NRS = (component.selectedRows.map((element: any) => ({ NRLOCALCLIENTE: element.NRLOCALCLIENTE, IDTIPOLOCALCLI: element.IDTIPOLOCALCLI })));
      const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-clientes');

      LoadingService.show();
      await Http.post('/inativarLocais', {
        TIPO1: NRS.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '1').map((elemento: any) => elemento.NRLOCALCLIENTE),
        TIPO2: NRS.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '2').map((elemento: any) => elemento.NRLOCALCLIENTE),
        TIPO3: NRS.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '3').map((elemento: any) => elemento.NRLOCALCLIENTE),
      }).then((response: IDictionary) => {
        if (response.data === 'Ok') {
          Utilities.showSuccessMessage();
          component.datasource.get();
          setTimeout(() => {
              if (component.datasource.data.length === 0) {
                cards.datasource.get();
              }
          }, 2000);
        } else {
          Utilities.showDialog('Atenção!', 'warning', 'Alguns locais não puderam ser inativados, pois possuem filhos ativos');
        }
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }

    // Integração com SM

    public async abrirModalConfigurarLocais() {
      this.modalConfigurarLocais = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/Fm_00002_000_configurar_locais', true);
      this.modalConfigurarLocais.show();
    }

    public fecharModalConfigurarLocais() {
      this.modalConfigurarLocais.hide();
      this.modalConfigurarLocais?.destroy();
      this.visibilidadeGridConfigurarLocais = false;
    }

    public changeContratos({ component }: any) {
      const selectLocais = Metadata.getInstance<SelectMultiple>('locais-sm');
      if (component.value.length > 0) {
        selectLocais.disabled = false;
        selectLocais.datasource.setFilter({
          NRSEQCONTRATO: component.value,
        });
      } else if (component.value.length < 1) {
        selectLocais.disabled = true;
        this.visibilidadeGridConfigurarLocais = false;
      }
    }

    public changeLocais({ component }: any) {
      if (component.value.length < 1) {
        this.visibilidadeGridConfigurarLocais = false;
      } else {
        this.visualizarLocais();
      }
    }

    public visualizarLocais() {
      const grid = Metadata.getInstance<TekGrid>('tek-grid-configurar-locais');
      const form = Metadata.getInstance<Form>('form-configurar-locais');
      const selectLocais = Metadata.getInstance<SelectMultiple>('locais-sm');

      if (form.validate()) {
        this.visibilidadeGridConfigurarLocais = true;
        grid.datasource.data = [];
        this.preencherGrid({ component: grid }, selectLocais.selectValue);
      } else {
        Utilities.alertMessage('Preencha os Locais para prosseguir com a configuração', 'error');
      }
    }

    public preencherGrid({ component }: any, locais: any) {
        locais.forEach((value: any) => {
          component.datasource.data.push({
            NRRELACESTRUTCONT: value.NRRELACESTRUTCONT,
            NMESTRUTURAM: value.NMESTRUTURAM,
            NRESTRUTURAM: value.NRESTRUTURAM,
            NRSEQCONTRATO: value.NRSEQCONTRATO,
            NRTIPOLOCALCLI: null,
            NMABREVLOCALCLI: null,
            DSCODIGOLOCALCLI: null,
            DSCORIDLOCAL: '#FF0000FF',
          });
        });
    }

    public salvarGridLocais({ component, event }: any) {
      event.preventDefault();

      if (component.isGridValid(component)) {
        component.saveEditedRows();
        component.cancelEditedRows();
      }
    }

    public salvarLocalSm() {
      const grid = Metadata.getInstance<TekGrid>('tek-grid-configurar-locais');
      const form = Metadata.getInstance<Form>('form-configurar-locais');
      const { data } = grid.datasource;
      const valores = data.filter((item: any) => item.NRTIPOLOCALCLI !== null);
      const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-clientes');

      if (form.validate()) {
        if (valores.length !== 0) {
          Http.post('/salvarLocalSm', grid.datasource.data).then((response: IDictionary) => {
            if (response.data.length > 0) {
              Utilities.showDialog('Local Existente', 'warning', '<br/>Locais já existentes! Não é possível inserir novamente: <br/><br/>'.concat(response.data.join('<br/>')));
            }
            cards.datasource.get();
            this.fecharModalConfigurarLocais();
            Utilities.showSuccessMessage();
          }).catch((error: any) => {
            Utilities.showErrorMessage(error);
          }).finally(() => {
            LoadingService.hide();
          });
        } else {
          Utilities.alertMessage('Preencha o Tipo de Local dos locais escolhidos para prosseguir!', 'error');
        }
      } else {
        Utilities.alertMessage('Preencha os Locais para prosseguir com a configuração', 'error');
      }
    }

    // const mensagem = '<br/>Os seguintes contratos não podem ser agrupados pois estão parametrizados para gerar parcelas separadas por local de atendimento: <br/><br/>' + contratosNaoAgrupam.join('<br/>");
    // this.dialogMessage('Operação bloqueada!', mensagem);

    /* public setCodMascara() {
    const dados: any = JSON.parse(<string>window.localStorage.getItem('padraoMascara')) || [];
    if (dados && dados.IDUTILIZAMASCCDLOCAT === 'S') {
      let masc = dados.DSMASCCDLOCAT;
      // const values: any = [
      //   ...new Set(
      //     (dados.DSMASCCDLOCAT.split(']').filter((elemento: any) => elemento.includes('[')))
      //       .map((elemento: any) => elemento.slice(elemento.indexOf('[') + 1))
      //       .map((elemento: any) => elemento.replace(elemento.substring(elemento.indexOf('{'), elemento.indexOf('}') + 1), '')),
      //   ).,
      // ];
      if (window.location.pathname === '/localAtendimento/edit') {
        const valuesBanco = this.dadosForUpdate || JSON.parse(<string>window.localStorage.getItem('dadosToEdit')) || [];
        masc = masc.replace('[SEQ]', valuesBanco.NRLOCALCLIENTE);
      }
      let component: any = Metadata.getInstance<Select>('sectorLocalAt');
      masc = masc.replace('[SETOR]', (component.value) ? component.selectedData.NMSETOR : '[SETOR]');
      component = Metadata.getInstance<TextInput>('abreviaturaLocalAt');
      masc = masc.replace('[NMABLOC]', (component.value) ? component.value : '[NMABLOC]');
      component = Metadata.getInstance<Select>('localParentLocalAt');
      if (!component.disabled) {
        // component.validate('required');
        if (component.value && ((masc.includes('[NMABLOCS]') && !component.value.NMABREVLOCALCLI) || (masc.includes(`[NMABLOCSALL{${masc.substring(masc.indexOf('{') + 1, masc.indexOf('}'))}}]`) && !component.value.SUP))) {
          DialogService.show({
            name: 'Conflito',
            type: 'info',
            title: 'Conflito encontrado',
            text: 'Mnemônico não pode ser substituído pois local superior não possui abreviação.<br>Edite o local superior para que possa ser criado uma dependência',
          });
          component.setValue();
        } else {
          if (component.value) {
            masc = masc.replace('[NMABLOCS]', component.value.NMABREVLOCALCLI);
            masc = masc.replace(`[NMABLOCSALL{${masc.substring(masc.indexOf('{') + 1, masc.indexOf('}'))}}]`, component.value.SUP.split(',').join(masc.substring(masc.indexOf('{') + 1, masc.indexOf('}'))));
          }
          Metadata.getInstance<TextInput>('codeLocalAt').value = masc;
        }
      } else {
        // component.removeValidation('required');
        masc = masc.replace('[NMABLOCS]', '');
        masc = masc.replace(`[NMABLOCSALL{${masc.substring(masc.indexOf('{') + 1, masc.indexOf('}'))}}]`, '');
        Metadata.getInstance<TextInput>('codeLocalAt').value = masc;
      }
    }
  } */

  /* public async codigo({ component }: any) {
    let dados: any = JSON.parse(<string>window.localStorage.getItem('padraoMascara')) || [];
    if (Object.keys(dados).length === 0) {
      await Http.get('/paramMascara').then((response: IDictionary) => {
        window.localStorage.setItem('padraoMascara', JSON.stringify(response.data[0]));
        [dados] = response.data;
      });
    }
    if (dados.IDOBRIGACDLOCAT === 'S') {
      component.addValidation('required');
      component.cssClass += ' zd-input-required';
    } else {
      component.removeValidation('required');
    }
    if (dados.IDUTILIZAMASCCDLOCAT === 'S') {
      const fieldAbrevi = Metadata.getInstance<TextInput>('abreviaturaLocalAt');
      fieldAbrevi.addValidation('required');
      fieldAbrevi.cssClass += ' zd-input-required';
    }
    component.disabled = (dados.IDUTILIZAMASCCDLOCAT === 'S' && dados.IDALTERAMASCCDLOCAT === 'N');
  } */
}
