/* eslint-disable */
import {
    Http, IDictionary,IEventParam, Metadata,
} from '@zeedhi/core';
import {
    Button, DialogService, Form, Grid, LoadingService, Modal, ModalService, Select, Textarea, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ServicosController {
    private basicModal!: Modal;
    
    public dispararFiltro({ component }: IEventParam<TekGrid>) {
        const filterButton = component.getAtomInstance<Button>('FILTER_BUTTON');
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public async exibirModal() {
      this.basicModal = await ModalService.createFromJson('servicos/modal/modal-insercao-servico', true);
      this.basicModal.show();
    }
  
    public fecharModal() {
      this.basicModal.hide();
      this.basicModal?.destroy();
    }

    public async abrirModoInsercao() {
        await this.exibirModal();
        const prodFaturamento = Metadata.getInstance<Select>('prod-faturamento');
        prodFaturamento.isVisible = true;
        prodFaturamento.addValidation('required', {});
        Metadata.getInstance<Select>('idativo').disabled = true;
    }

    public async abrirModoEdicao({ row } : any) {
        await this.exibirModal();
        console.log(row);
        const prodFaturamento = Metadata.getInstance<Select>('prod-faturamento');
        const ativo = Metadata.getInstance<Select>('idativo');

        Metadata.getInstance<TextInput>('cod-servico').value = row.CDSERVORCA;
        Metadata.getInstance<TextInput>('nome-servico').value = row.NMSERVORCA;
        prodFaturamento.isVisible = false;
        prodFaturamento.clearValidations();
        Metadata.getInstance<Textarea>('descricao-servico').value = row.DSSERVORCA;
        ativo.value = row.IDATIVO;
        ativo.disabled = false;
    }

    public inserirServico() {
        const form = Metadata.getInstance<Form>('form').validate();
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid');
       
        if (form) {
            LoadingService.show();
            Http.post('/inserirServico', {
                CDSERVORCA: Metadata.getInstance<TextInput>('cod-servico').value,
                NMSERVORCA: Metadata.getInstance<TextInput>('nome-servico').value,
                DSSERVORCA: Metadata.getInstance<Textarea>('descricao-servico').value, 
                CDPRODUTO: Metadata.getInstance<Select>('prod-faturamento').value,
                IDATIVO: Metadata.getInstance<Select>('idativo').value,
            }).then(() => {
               this.fecharModal();
               grid.datasource.get();
               Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if(error.response.data.error === 'Exception: IDATIVO != S') {
                    Utilities.alertMessage('Não foi possível atualizar este Serviço pois ele está ativo', 'error');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios (*)', 'error');
        }
    }

    public dialogDeletarServicos({ component, event }: any) {
        event.preventDefault();
  
        DialogService.show({
          name: 'dialog-deletar',
          type: 'warning',
          title: 'Atenção',
          text: "Todos os dados deste(s) serviço(s) serão perdidos. Deseja continuar?",
          buttons: [
            {
              name: 'NAO',
              component: 'ZdButton',
              label: 'Não',
              outline: true,
              events: {
                click: () => {
                  DialogService.hide();
                },
              },
            },
            {
              name: 'sim',
              component: 'ZdButton',
              label: 'Sim',
              events: {
                click: () => {
                  this.deletarServicos({ component: component});
                  DialogService.hide();
                }
              },
            },
          ],
        });
    }

    public deletarServicos({ component }: any) {
        const NRS = (component.selectedRows.map((element: any) => element.CDSERVORCA));
        
        LoadingService.show();
        Http.post('/deletarServicos', {
            NRS: NRS,
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
  
}